'use strict';

Object.defineProperty(exports, Symbol.toStringTag, { value: 'Module' });

const qwik = require('@builder.io/qwik');
const jsxRuntime = require('@builder.io/qwik/jsx-runtime');
const build = require('@builder.io/qwik/build');
const core = require('@angular/core');
const platformServer = require('@angular/platform-server');
const platformBrowser = require('@angular/platform-browser');
const common = require('@angular/common');
const slot = require('./slot.cjs');
const rxjs = require('rxjs');

const SLOT_MARK = "SLOT";
const SLOT_COMMENT = `<!--${SLOT_MARK}-->`;
const projectableNodesMap = /* @__PURE__ */ new Set();
const create = core["ɵRender3ComponentFactory"].prototype.create;
core["ɵRender3ComponentFactory"].prototype.create = function(injector, projectableNodes, rootSelectorOrNode, environmentInjector) {
  if (projectableNodesMap.has(this.componentType)) {
    const document_local = this["ngModule"].injector.get(common.DOCUMENT);
    const slotComment = document_local.createComment(SLOT_MARK);
    projectableNodes = [
      [
        slotComment
      ]
    ];
  }
  return create.call(this, injector, projectableNodes, rootSelectorOrNode, environmentInjector);
};
const QWIK_ANGULAR_STATIC_PROPS = new core.InjectionToken("@builder.io/qwik-angular: Static Props w/ Mirror Provider", {
  factory() {
    return {
      props: {},
      mirror: {}
    };
  }
});
const STATIC_PROPS_HOOK_PROVIDER = {
  provide: platformServer.BEFORE_APP_SERIALIZED,
  useFactory: (appRef, { props, mirror }) => {
    return () => {
      const compRef = appRef.components[0];
      if (compRef && props && mirror) {
        for (const [key, value] of Object.entries(props))
          if (
            // we double-check inputs on ComponentMirror
            // because there might be additional props
            // that aren't actually Input defined on the Component
            mirror.inputs.some(({ templateName, propName }) => templateName === key || propName === key)
          )
            compRef.setInput(key, value);
        compRef.changeDetectorRef.detectChanges();
      }
    };
  },
  deps: [
    core.ApplicationRef,
    QWIK_ANGULAR_STATIC_PROPS
  ],
  multi: true
};
class MockApplicationRef extends core.ApplicationRef {
  constructor() {
    super(...arguments);
    this.isStable = new rxjs.BehaviorSubject(true);
  }
}
async function renderFromServer(Host, angularCmp$, hostRef, slotRef, props) {
  if (build.isServer) {
    const component = await angularCmp$.resolve();
    const mirror = core.reflectComponentType(component);
    if (mirror?.ngContentSelectors.length)
      projectableNodesMap.add(component);
    const appId = mirror?.selector || component.name.toString().toLowerCase();
    const document = `<${appId}></${appId}>`;
    const mockZoneProviders = [
      {
        provide: core.ApplicationRef,
        useFactory: () => new MockApplicationRef()
      },
      {
        provide: core.NgZone,
        useClass: core["ɵNoopNgZone"]
      }
    ];
    const bootstrap = () => platformBrowser.bootstrapApplication(component, {
      providers: [
        ...mockZoneProviders,
        {
          provide: QWIK_ANGULAR_STATIC_PROPS,
          useValue: {
            props,
            mirror
          }
        },
        STATIC_PROPS_HOOK_PROVIDER,
        platformServer.provideServerRendering()
      ]
    });
    const html = await platformServer.renderApplication(bootstrap, {
      document
    });
    const index = html.indexOf(SLOT_COMMENT);
    if (index > 0) {
      const part1 = html.slice(0, index);
      const part2 = html.slice(index + SLOT_COMMENT.length);
      return /* @__PURE__ */ qwik._jsxC(Host, {
        ref: hostRef,
        ...slot.getHostProps(props),
        children: [
          /* @__PURE__ */ qwik._jsxC(qwik.SSRRaw, {
            data: part1
          }, 3, "wl_0"),
          /* @__PURE__ */ qwik._jsxQ("q-slot", {
            ref: slotRef
          }, {
            projected: true
          }, /* @__PURE__ */ qwik._jsxC(qwik.Slot, null, 3, "wl_1"), 1, null),
          /* @__PURE__ */ qwik._jsxC(qwik.SSRRaw, {
            data: part2
          }, 3, "wl_2")
        ]
      }, 0, "wl_3");
    }
    return /* @__PURE__ */ qwik._jsxC(jsxRuntime.Fragment, {
      children: [
        /* @__PURE__ */ qwik._jsxC(Host, {
          ref: hostRef,
          ...slot.getHostProps(props),
          children: /* @__PURE__ */ qwik._jsxC(qwik.SSRRaw, {
            data: html
          }, 3, "wl_4")
        }, 0, "wl_5"),
        /* @__PURE__ */ qwik._jsxQ("q-slot", {
          ref: slotRef
        }, null, /* @__PURE__ */ qwik._jsxC(qwik.Slot, null, 3, "wl_6"), 1, null)
      ]
    }, 1, "wl_7");
  }
  return null;
}

exports.renderFromServer = renderFromServer;
