import { _jsxC, SSRRaw, _jsxQ, Slot } from '@builder.io/qwik';
import { Fragment } from '@builder.io/qwik/jsx-runtime';
import { isServer } from '@builder.io/qwik/build';
import { ɵRender3ComponentFactory, InjectionToken, reflectComponentType, ApplicationRef, NgZone, ɵNoopNgZone } from '@angular/core';
import { renderApplication, provideServerRendering, BEFORE_APP_SERIALIZED } from '@angular/platform-server';
import { bootstrapApplication } from '@angular/platform-browser';
import { DOCUMENT } from '@angular/common';
import { getHostProps } from './slot.mjs';
import { BehaviorSubject } from 'rxjs';

const SLOT_MARK = "SLOT";
const SLOT_COMMENT = `<!--${SLOT_MARK}-->`;
const projectableNodesMap = /* @__PURE__ */ new Set();
const create = ɵRender3ComponentFactory.prototype.create;
ɵRender3ComponentFactory.prototype.create = function(injector, projectableNodes, rootSelectorOrNode, environmentInjector) {
  if (projectableNodesMap.has(this.componentType)) {
    const document_local = this["ngModule"].injector.get(DOCUMENT);
    const slotComment = document_local.createComment(SLOT_MARK);
    projectableNodes = [
      [
        slotComment
      ]
    ];
  }
  return create.call(this, injector, projectableNodes, rootSelectorOrNode, environmentInjector);
};
const QWIK_ANGULAR_STATIC_PROPS = new InjectionToken("@builder.io/qwik-angular: Static Props w/ Mirror Provider", {
  factory() {
    return {
      props: {},
      mirror: {}
    };
  }
});
const STATIC_PROPS_HOOK_PROVIDER = {
  provide: BEFORE_APP_SERIALIZED,
  useFactory: (appRef, { props, mirror }) => {
    return () => {
      const compRef = appRef.components[0];
      if (compRef && props && mirror) {
        for (const [key, value] of Object.entries(props))
          if (
            // we double-check inputs on ComponentMirror
            // because there might be additional props
            // that aren't actually Input defined on the Component
            mirror.inputs.some(({ templateName, propName }) => templateName === key || propName === key)
          )
            compRef.setInput(key, value);
        compRef.changeDetectorRef.detectChanges();
      }
    };
  },
  deps: [
    ApplicationRef,
    QWIK_ANGULAR_STATIC_PROPS
  ],
  multi: true
};
class MockApplicationRef extends ApplicationRef {
  constructor() {
    super(...arguments);
    this.isStable = new BehaviorSubject(true);
  }
}
async function renderFromServer(Host, angularCmp$, hostRef, slotRef, props) {
  if (isServer) {
    const component = await angularCmp$.resolve();
    const mirror = reflectComponentType(component);
    if (mirror?.ngContentSelectors.length)
      projectableNodesMap.add(component);
    const appId = mirror?.selector || component.name.toString().toLowerCase();
    const document = `<${appId}></${appId}>`;
    const mockZoneProviders = [
      {
        provide: ApplicationRef,
        useFactory: () => new MockApplicationRef()
      },
      {
        provide: NgZone,
        useClass: ɵNoopNgZone
      }
    ];
    const bootstrap = () => bootstrapApplication(component, {
      providers: [
        ...mockZoneProviders,
        {
          provide: QWIK_ANGULAR_STATIC_PROPS,
          useValue: {
            props,
            mirror
          }
        },
        STATIC_PROPS_HOOK_PROVIDER,
        provideServerRendering()
      ]
    });
    const html = await renderApplication(bootstrap, {
      document
    });
    const index = html.indexOf(SLOT_COMMENT);
    if (index > 0) {
      const part1 = html.slice(0, index);
      const part2 = html.slice(index + SLOT_COMMENT.length);
      return /* @__PURE__ */ _jsxC(Host, {
        ref: hostRef,
        ...getHostProps(props),
        children: [
          /* @__PURE__ */ _jsxC(SSRRaw, {
            data: part1
          }, 3, "wl_0"),
          /* @__PURE__ */ _jsxQ("q-slot", {
            ref: slotRef
          }, {
            projected: true
          }, /* @__PURE__ */ _jsxC(Slot, null, 3, "wl_1"), 1, null),
          /* @__PURE__ */ _jsxC(SSRRaw, {
            data: part2
          }, 3, "wl_2")
        ]
      }, 0, "wl_3");
    }
    return /* @__PURE__ */ _jsxC(Fragment, {
      children: [
        /* @__PURE__ */ _jsxC(Host, {
          ref: hostRef,
          ...getHostProps(props),
          children: /* @__PURE__ */ _jsxC(SSRRaw, {
            data: html
          }, 3, "wl_4")
        }, 0, "wl_5"),
        /* @__PURE__ */ _jsxQ("q-slot", {
          ref: slotRef
        }, null, /* @__PURE__ */ _jsxC(Slot, null, 3, "wl_6"), 1, null)
      ]
    }, 1, "wl_7");
  }
  return null;
}

export { renderFromServer };
