'use strict';

Object.defineProperty(exports, Symbol.toStringTag, { value: 'Module' });

const originalAngularPluginsImport = require('@analogjs/vite-plugin-angular');
const sass = require('sass');

const originalAngularPlugins = originalAngularPluginsImport.default;
const ANALOG_ANGULAR_PLUGIN = "@analogjs/vite-plugin-angular";
function angular(options) {
  const plugins = originalAngularPlugins(options);
  for (const p of plugins)
    if (p.name === ANALOG_ANGULAR_PLUGIN) {
      const transform = p.transform;
      p.transform = function(code, id, ssrOpts) {
        if (!id.includes(options.componentsDir))
          return;
        return transform.call(this, code, id, ssrOpts);
      };
    }
  return [
    ...plugins,
    analogQwikPlugin(options)
  ];
}
function analogQwikPlugin(options) {
  const bundleSassFilePaths = options.bundleSassFilesInDevMode?.paths;
  let viteCommand = "serve";
  const vitePluginQwikAngular = {
    name: "vite-plugin-qwik-angular",
    config(viteConfig, viteEnv) {
      viteCommand = viteEnv.command;
      return {
        optimizeDeps: {
          include: [
            "@angular/core",
            "@angular/platform-browser",
            "@angular/platform-browser/animations",
            "@angular/compiler",
            "@angular/common",
            "@angular/animations",
            "@angular/animations/browser"
          ],
          exclude: [
            "@angular/platform-server"
          ]
        }
      };
    },
    load: function(id) {
      if (viteCommand === "serve" && bundleSassFilePaths?.some((p) => id.includes(p))) {
        id = id.replace(/\?(.*)/, "");
        try {
          const compiledAsset = sass.compile(id, options.bundleSassFilesInDevMode?.compileOptions).css;
          return compiledAsset;
        } catch (e) {
        }
      }
      return;
    }
  };
  return vitePluginQwikAngular;
}

exports.angular = angular;
